==> odoo_19.0.20251011.zip: odoo-19.0.post20251011/odoo/addons/mass_mailing/static/tests/mass_mailing_html_field.test.js <==
import { expect, test, describe, beforeEach } from "@odoo/hoot";
import {
    defineModels,
    fields,
    models,
    mountView,
    onRpc,
    clickSave,
    patchWithCleanup,
} from "@web/../tests/web_test_helpers";
import { click, queryOne, waitFor } from "@odoo/hoot-dom";
import { defineMailModels } from "@mail/../tests/mail_test_helpers";
import { unmockedOrm } from "@web/../tests/_framework/module_set.hoot";
import { MassMailingIframe } from "../src/iframe/mass_mailing_iframe";

class Mailing extends models.Model {
    _name = "mailing.mailing";

    display_name = fields.Char();
    subject = fields.Char();
    body_arch = fields.Html();
    body_html = fields.Html();
    mailing_model_id = fields.Many2one({ relation: "ir.model", string: "Recipients" });
    mailing_model_name = fields.Char({ string: "Recipients Model Name" });

    action_fetch_favorites() {
        return [];
    }

    _records = [
        {
            id: 1,
            display_name: "Belgian Event promotion",
            mailing_model_id: 1,
        },
    ];
}

class IrUiView extends models.Model {
    async render_public_asset(template, values) {
        return unmockedOrm("ir.ui.view", "render_public_asset", [template, values], {});
    }
}

class IrModel extends models.Model {
    _name = "ir.model";

    name = fields.Char();
    display_name = fields.Char();
    model = fields.Char();

    _records = [
        {
            id: 1,
            name: "Event",
            display_name: "Event",
            model: "event",
        },
    ];
}

class Event extends models.Model {
    _name = "event";

    name = fields.Char();
    country = fields.Char();

    _records = [{ id: 1, name: "BE Event", country: "be" }];
}

defineMailModels();
defineModels([IrModel, IrUiView, Mailing, Event]);

const mailViewArch = `
<form>
    <field name="mailing_model_name" invisible="1"/>
    <field name="mailing_model_id" invisible="1"/>
    <field name="body_html" invisible="1"/>
    <field name="body_arch" class="o_mail_body_mailing" widget="mass_mailing_html"
        options="{ 'inline_field': 'body_html' }"/>
</form>
`;

describe.current.tags("desktop");
describe("field HTML", () => {
    beforeEach(() => {
        patchWithCleanup(MassMailingIframe.prototype, {
            // Css assets are not needed for these tests.
            loadIframeAssets() {},
        });
    });
    test("save arch and html", async () => {
        onRpc("web_save", ({ args }) => {
            expect(args[1].body_arch).toMatch(/^<div/);
            expect(args[1].body_html).toMatch(/^<table/);
            expect.step("web_save mail body");
        });
        await mountView({
            type: "form",
            resModel: "mailing.mailing",
            resId: 1,
            arch: mailViewArch,
        });
        expect(queryOne(".o_mass_mailing_iframe_wrapper iframe")).toHaveClass("d-none");
        await click(waitFor(".o_mailing_template_preview_wrapper a:contains(Start From Scratch)"));
        await waitFor(".o_mass_mailing_iframe_wrapper iframe:not(.d-none)");
        expect(await waitFor(":iframe .o_layout", { timeout: 3000 })).toHaveClass("o_empty_theme");
        await clickSave();
        await expect.waitForSteps(["web_save mail body"]);
    });
});

==> odoo_19.0.20251018.zip: odoo-19.0.post20251018/odoo/addons/mass_mailing/static/tests/mass_mailing_html_field.test.js <==
import { expect, test, describe, beforeEach } from "@odoo/hoot";
import {
    defineModels,
    fields,
    models,
    mountView,
    onRpc,
    clickSave,
    patchWithCleanup,
} from "@web/../tests/web_test_helpers";
import { click, queryOne, waitFor } from "@odoo/hoot-dom";
import { defineMailModels } from "@mail/../tests/mail_test_helpers";
import { unmockedOrm } from "@web/../tests/_framework/module_set.hoot";
import { MassMailingIframe } from "../src/iframe/mass_mailing_iframe";

class Mailing extends models.Model {
    _name = "mailing.mailing";

    display_name = fields.Char();
    subject = fields.Char();
    body_arch = fields.Html();
    body_html = fields.Html();
    mailing_model_id = fields.Many2one({ relation: "ir.model", string: "Recipients" });
    mailing_model_name = fields.Char({ string: "Recipients Model Name" });

    action_fetch_favorites() {
        return [];
    }

    _records = [
        {
            id: 1,
            display_name: "Belgian Event promotion",
            mailing_model_id: 1,
        },
        {
            id: 2,
            display_name: "Sent Belgian Event promotion",
            mailing_model_id: 1,
            body_arch: `
                <div data_name="Mailing" class="o_layout oe_unremovable oe_unmovable o_empty_theme">
                    <div class="container o_mail_wrapper o_mail_regular oe_unremovable">
                        <div class="row">
                            <div class="col o_mail_no_options o_mail_wrapper_td bg-white oe_structure o_editable oe_empty" data-editor-message-default="true" data-editor-message="DRAG BUILDING BLOCKS HERE" contenteditable="true">
                                This element <t t-out="'should be inline'"/>
                            </div>
                        </div>
                    </div>
                </div>
            `,
        },
    ];
}

class IrUiView extends models.Model {
    async render_public_asset(template, values) {
        return unmockedOrm("ir.ui.view", "render_public_asset", [template, values], {});
    }
}

class IrModel extends models.Model {
    _name = "ir.model";

    name = fields.Char();
    display_name = fields.Char();
    model = fields.Char();

    _records = [
        {
            id: 1,
            name: "Event",
            display_name: "Event",
            model: "event",
        },
    ];
}

class Event extends models.Model {
    _name = "event";

    name = fields.Char();
    country = fields.Char();

    _records = [{ id: 1, name: "BE Event", country: "be" }];
}

defineMailModels();
defineModels([IrModel, IrUiView, Mailing, Event]);

const mailViewArch = `
<form>
    <field name="mailing_model_name" invisible="1"/>
    <field name="mailing_model_id" invisible="1"/>
    <field name="body_html" invisible="1"/>
    <field name="body_arch" class="o_mail_body_mailing" widget="mass_mailing_html"
        options="{ 'inline_field': 'body_html' }"/>
</form>
`;

const readonlyMailViewArch = `
<form>
    <field name="mailing_model_name" invisible="1"/>
    <field name="mailing_model_id" invisible="1"/>
    <field name="body_html" invisible="1"/>
    <field name="body_arch" class="o_mail_body_mailing" widget="mass_mailing_html"
        options="{
            'inline_field': 'body_html',
            'dynamic_placeholder': true,
        }" readonly="true"/>
</form>
`;

describe.current.tags("desktop");
describe("field HTML", () => {
    beforeEach(() => {
        patchWithCleanup(MassMailingIframe.prototype, {
            // Css assets are not needed for these tests.
            loadIframeAssets() {},
        });
    });
    test("save arch and html", async () => {
        onRpc("web_save", ({ args }) => {
            expect(args[1].body_arch).toMatch(/^<div/);
            expect(args[1].body_html).toMatch(/^<table/);
            expect.step("web_save mail body");
        });
        await mountView({
            type: "form",
            resModel: "mailing.mailing",
            resId: 1,
            arch: mailViewArch,
        });
        expect(queryOne(".o_mass_mailing_iframe_wrapper iframe")).toHaveClass("d-none");
        await click(waitFor(".o_mailing_template_preview_wrapper a:contains(Start From Scratch)"));
        await waitFor(".o_mass_mailing_iframe_wrapper iframe:not(.d-none)");
        expect(await waitFor(":iframe .o_layout", { timeout: 3000 })).toHaveClass("o_empty_theme");
        await clickSave();
        await expect.waitForSteps(["web_save mail body"]);
    });
    test("t-out field in uneditable mode inline", async () => {
        await mountView({
            type: "form",
            resModel: "mailing.mailing",
            resId: 2,
            arch: readonlyMailViewArch,
        });
        await waitFor(".o_mass_mailing_iframe_wrapper iframe:not(.d-none)");
        const tElement = await waitFor(":iframe t", { timeout: 3000 });

        // assert that we are in readonly mode (sanity check)
        expect(":iframe .o_mass_mailing_value.o_readonly").toHaveCount(1);

        // assert that tElement style has inline attibute
        expect(tElement).toHaveAttribute("data-oe-t-inline", "true");
    });
});
